//CONSTANTS

// 10MB file limit
var MAX_FILE_SIZE = 10485760;

// The list of current files, accessible in global scope
var CURRENT_FILES = {};

// The parent element of story point attachments
var PREVIEW_TARGET = 'cur-point-attachments';

// Image location for undefined attachments
var UNDEFINED_ATTACHMENT_URL = 'images/undefined_image.png';

/**
 * Synchronize the values between given inputs
 *
 * @param {string} source the invoking input ID or some other jquery compatible selector
 * @param {array} inputs input IDs or some other jquery compatible selectors
 */
function synchronizeInputs(source, inputs) {
  var targets = $(inputs.join(', '));

  targets.val($(source).val());
}

/**
 * Delete files from an input ID'd similarly to the ID of the invoking element
 * Invokes synchronizeFileInputs() afterwards
 *
 * @param {event} e html event object
 * @param {string} id the input name (sans # or other identifiers)
 */
function deleteFileFromInput(e, id = null) {
  var fileInputID = id;
  if (fileInputID === null) {
    e.preventDefault();
    fileInputID = e.target.id.replace('Delete', '');
  }

  // Delete the file
  var $file = $('#' + fileInputID);
  $file.val('');

  // Re-sync all files
  synchronizeFileInputs();
}

/**
 * Synchronize (hardcoded) file inputs with the DOM
 *
 * - Organize all selected files
 * - Display Image previews above corresponding file input
 * - Display Video and Audio previews in appropriate players
 *   above the corresponding input
 * - Copy filenames into 'preview' input on final form display page
 * - Enable delete buttons below file inputs with files in them
 * - Pull files into constant for access in submission
 */
function synchronizeFileInputs() {
  var $files = $('#fileInput1, #fileInput2, #fileInput3');

  // Pull all files from their inputs, organize them by input
  var files = {};
  for (file in $files) {
    switch(file) {
      case '0':
        files['fileInput1'] = null;
        if ($files[file].files != null && $files[file].files.length) {
          files['fileInput1'] = $files[file].files[0];
        }
        break;
      case '1':
        files['fileInput2'] = null;
        if ($files[file].files != null && $files[file].files.length) {
          files['fileInput2'] = $files[file].files[0];
        }
        break;
      case '2':
        files['fileInput3'] = null;
        if ($files[file].files != null && $files[file].files.length) {
          files['fileInput3'] = $files[file].files[0];
        }
        break;
      default:
        break;
    }
  }

  var filenames = [];
  for(input in files) {
    if (files[input] != null && files[input].size > MAX_FILE_SIZE) {
      alert('File must be smaller than 10MB!');
      deleteFileFromInput(null, input);
    }

    // Preview files in the file selector preview windows
    // Images should be displayed as is, audio and video should be passed to the respective handlers
    var imgID = '#' + input + 'Img';
    var videoID = '#' + input + 'Video';
    var audioID = '#' + input + 'Audio';
    if (files[input] != null) {
      var type = files[input].type;
      // This is repetitive but it works
      // For every type of file, inject it into appropriate container
      // Hide other containers that are no longer being used
      if (type.includes('image')) {
        $(imgID)[0].src = window.URL.createObjectURL(files[input]);
        $(imgID).show();
        $(audioID).hide()[0].src = '';
        $(videoID).hide()[0].src = '';
      } else if (type.includes('video')){
        $(imgID).hide()[0].src = UNDEFINED_ATTACHMENT_URL;
        $(audioID).hide()[0].src = '';
        injectVideoPlayer(files[input], videoID);
      } else if (type.includes('audio')) {
        $(imgID).hide()[0].src = UNDEFINED_ATTACHMENT_URL;
        $(videoID).hide()[0].src = '';
        injectAudioPlayer(files[input], audioID);
      } else {
        // The file is of an unsupported type, 'delete' it
        file[input] = null;
      }
    } else {
      // Clear all input sources, show image element, hide audio/video
      $(imgID).show()[0].src = UNDEFINED_ATTACHMENT_URL;
      $(videoID).hide()[0].src = '';
      $(audioID).hide()[0].src = '';
    }

    // Use the input names from files.keys() to generate the ID of the name field
    var id = '#' + input + 'Name';
    $(id).text(
      files[input] !== null
        ? files[input].name
        : 'No attachment selected'
    )

    // Track file names for the filePreviewInput
    files[input] != null
      ? filenames.push(files[input].name)
      : null;

    // Enable the delete button for file selectors who contain files
    var id = '#' + input + 'Delete';
    files[input] !== null
      ? $(id).removeClass('disabled')
      : $(id).addClass('disabled');
  }

  // Sync filenames into the filePreviewInput
  $('#fileNamePreviewInput').val(
    filenames.length
      ? filenames.join(', ')
      : 'No attachments selected'
  );

  CURRENT_FILES = files;
}

/**
 * Generate preview elements for story attachments, inject content into them
 *
 * @param {array} attachments file attachments downloaded from arcGIS
 */
function synchronizeAttachmentPreviews(attachments) {
  for (var i = 0; i < attachments.length; i++) {
    // Pull out relevant attachment information
    var targetID = 'attachment' + i;
    var type = attachments[i].contentType;
    var filename = attachments[i].name;
    var url = attachments[i].url;
    var helperText = function(type) {
      if (type.includes('image')) {
        return 'Click to enlarge image';
      } else if (type.includes('video')) {
        return 'Use player controls to enable fullscreen video';
      } else if (type.includes('audio')) {
        return 'Use player controls to increase or decrease volume';
      }

      return '';
    }

    // Determine the type of element to generate, generate it
    if (type.includes('image')) {
      generateImageElement(PREVIEW_TARGET, targetID);

      // Attach a click event to images that allows them to be expanded into a modal
      $('#' + targetID).on("click", function () {
        $("#modal-enlarged-image").attr("src", url);
        $("#modal-enlarged-image").attr("src", $(this).attr("src"));
        $("#image-modal-label").text(filename + 'preview');
        $("#image-modal").modal("show");
      });
    } else if (type.includes('video')) {
      generateVideoElement(PREVIEW_TARGET, targetID);
    } else if (type.includes('audio')) {
      generateAudioElement(PREVIEW_TARGET, targetID);
    }

    // Prepend the attachment's helper text above the file
    $('#' + targetID).parent().prepend($('<small>').text(helperText(type)).append($('<br />')));

    // Inject the attachment url into the generated element's src attribute
    $('#' + targetID)[0].src = attachments[i].url;

    if (type.includes('video')) {
      $('#' + targetID)[0].load();
    }
  }
}

/**
 * Append a <img> element onto the element with 'parent' id
 *
 * @param {string} parent The ID of the desired parent element
 * @param {string} generatedID The ID that should be given to the generated element
 */
function generateImageElement(parent, generatedID) {
  $('#' + parent).append(
    $('<li>').append(
      $('<img>').attr('id', generatedID).addClass('usrImgSubmission clickable')
    )
  );
}

/**
 * Append a <audio> element onto the element with 'parent' id
 *
 * @param {string} parent The ID of the desired parent element
 * @param {string} generatedID The ID that should be given to the generated element
 */
function generateAudioElement(parent, generatedID) {
  $('#' + parent).append(
    $('<li>').append(
      $('<audio>').attr('id', generatedID).attr('controls', true)
    )
  );
}

/**
 * Append a <video> element onto the element with 'parent' id
 *
 * @param {string} parent The ID of the desired parent element
 * @param {string} generatedID The ID that should be given to the generated element
 */
function generateVideoElement(parent, generatedID) {
  $('#' + parent).append(
    $('<li>').append(
      $('<video>').attr('id', generatedID).attr('controls', true).addClass('usrImgSubmission')
      .attr('type', 'video/mp4')
    )
  );
}

/**
 * Inject a video player into the destination element
 * This function is used for raw file blobs, not URLs
 *
 * @param {file} file user's uploaded file
 * @param {string} player_id desired html ID of the player
 */
function injectVideoPlayer(file, player_id) {
  var videoURL = window.URL.createObjectURL(file);
  $(player_id).show()[0].src = videoURL;
  $(player_id)[0].load();
}

/**
 * Inject an audio player into the destination element
 * This function is used for raw file blobs, not URLs
 *
 * @param {file} file user's uploaded file
 * @param {string} player_id desired html ID of the player
 */
function injectAudioPlayer(file, player_id) {
  var audioURL = window.URL.createObjectURL(file);
  $(player_id).show()[0].src = audioURL;
  $(player_id)[0].load();
}
